Package: Rcollectl
Title: Help use collectl with R in Linux, to measure resource
        consumption in R processes
Version: 1.10.0
Date: 2023-03-08
Authors@R: 
    c(person(given = "Vincent",
           family = "Carey",
           role = c("aut", "cre"),
           email = "stvjc@channing.harvard.edu",
           comment = c(ORCID = "0000-0003-4046-0063")),
      person(given = "Yubo",
           family = "Cheng", 
           role = c("aut"),
           email="yubo.cheng@roswellpark.org"))
Description: Provide functions to obtain instrumentation data on
    processes in a unix environment.  Parse output of a collectl
    run.  Vizualize aspects of system usage over time, with annotation.
Imports: utils, ggplot2, lubridate, processx
License: Artistic-2.0
URL: https://github.com/vjcitn/Rcollectl
BugReports: https://support.bioconductor.org/t/Rcollectl
biocViews: Software, Infrastructure
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
SystemRequirements: collectl
Suggests: knitr, BiocStyle, knitcitations, sessioninfo, rmarkdown,
        testthat, covr
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/Rcollectl
git_branch: RELEASE_3_22
git_last_commit: 1a76401
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:58:04 UTC; biocbuild
Author: Vincent Carey [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4046-0063>),
  Yubo Cheng [aut]
Maintainer: Vincent Carey <stvjc@channing.harvard.edu>

# Generated by roxygen2: do not edit by hand

S3method(print,Rcollectl_process)
export(browse_units)
export(cl_exists)
export(cl_parse)
export(cl_result_path)
export(cl_start)
export(cl_stop)
export(cl_timestamp)
export(cl_timestamp_label)
export(cl_timestamp_layer)
export(plot_usage)
import(ggplot2)
importFrom(lubridate,as_datetime)
importFrom(processx,process)
importFrom(utils,browseURL)
importFrom(utils,read.delim)

<!-- badges: start -->

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://www.tidyverse.org/lifecycle/#experimental)

<!--
[![BioC
status](http://www.bioconductor.org/shields/build/release/bioc/Rcollectl.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/Rcollectl)
[![BioC dev
status](http://www.bioconductor.org/shields/build/devel/bioc/Rcollectl.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/Rcollectl)
-->
[![R build
status](https://github.com/vjcitn/Rcollectl/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/vjcitn/Rcollectl/actions)
[![Codecov test
coverage](https://codecov.io/gh/vjcitn/Rcollectl/branch/main/graph/badge.svg)](https://codecov.io/gh/vjcitn/Rcollectl?branch=main)
<!--
[![Support site activity, last 6 months: tagged questions/avg. answers
per question/avg. comments per question/accepted answers, or 0 if no
tagged
posts.](http://www.bioconductor.org/shields/posts/Rcollectl.svg)](https://support.bioconductor.org/t/Rcollectl/)
-->
[![GitHub
issues](https://img.shields.io/github/issues/vjcitn/Rcollectl)](https://github.com/vjcitn/Rcollectl/issues)
<!-- badges: end -->

# Rcollectl: simple interfaces to collectl output

Profiling R computations is supported by Rprof and profvis.  These measure memory consumption and
function execution counts and timings.  In workflow design we are also interested in CPU load, disk I/O, and network
traffic.  There is no standard portable approach to measuring these.  In this package, we focus
on linux systems measurement with
the [collectl](http://collectl.sourceforge.net/index.html) suite of tools.  


## Notes from the AnVIL workspace description

Our objective in this workspace is demonstration of instrumentation of
bioinformatic tasks.  By "instrumentation" we mean the definition and use
of tools that measure computational resource consumption: memory, disk, CPU, network traffic.

(On AnVIL, we do not know how to identify the network device and data on network traffic
are not obtained as of 1/1/2021.)

This Dashboard Description discusses details of an example using [salmon](https://github.com/COMBINE-lab/salmon) to
carry out quantification of an RNA-seq experiment.

## Basic resources

### Workspace backbone

This workspace was initialized using AnVILPublish.  The runtime/cloud environment used as of 2020-01-04 is

dockerhub vjcitn/instr:0.0.3

### Raw data

The FASTQ files underlying the "airway" RNA-seq workflow were collected using commands like

```
gsutil -u landmarkanvil2 ls  gs://sra-pub-run-3/SRR1039512/SRR1039512.1
gsutil -u landmarkanvil2 cp  gs://sra-pub-run-4/SRR1039513/SRR1039513.1 .
gsutil -u landmarkanvil2 cp  gs://sra-pub-run-4/SRR1039516/SRR1039516.1 .
gsutil -u landmarkanvil2 cp  gs://sra-pub-run-2/SRR1039517/SRR1039517.1 .
gsutil -u landmarkanvil2 cp  gs://sra-pub-run-2/SRR1039520/SRR1039520.1 .
gsutil -u landmarkanvil2 cp  gs://sra-pub-run-3/SRR1039521/SRR1039521.1 .
```

A public bucket with the extracted fastq was then produced via
```
fastq-dump --split-3 *512.1 &
fastq-dump --split-3 *513.1 &
fastq-dump --split-3 *516.1 &
fastq-dump --split-3 *517.1 &
fastq-dump --split-3 *520.1 &
fastq-dump --split-3 *521.1 &
gzip *fastq
gsutil ls gs://bioc-airway-fastq
gsutil cp SRR1039508.1_2.fastq.gz gs://bioc-airway-fastq/
```

## Quantification process

### Software and indexing resources

Our aim is to analyze the resource consumption in processing this sort of data with
R using Rcollectl.  We'll describe how to run salmon to generate quantifications,and
we will attempt to give estimates for processing various volumes of data by various
approaches.

We installed snakemake with pip3 and salmon via
```
wget https://github.com/COMBINE-lab/salmon/releases/download/v1.4.0/salmon-1.4.0_linux_x86_64.tar.gz
```


We obtained GENCODE transcript sequences:

```
wget ftp://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_human/release_36/gencode.v36.transcripts.fa.gz
```
And built the index `gencode.v36_salmon_1.4.0` via

```
salmon index --gencode -t gencode.v36.transcripts.fa.gz -i gencode.v36_salmon_1.4.0
```

### Snakemake for workflow definition

For a single sample (paired-end) we have a snakemake file:

```
rstudio@saturn-1f2f18e5-4182-40c0-8449-1f301b5c3b03:~$ cat snakemake_one
DATASETS = ["SRR1039508.1"]

SALMON = "/home/rstudio/bin/salmon"

rule all:
  input: expand("quants/{dataset}/quant.sf", dataset=DATASETS)

rule salmon_quant:
    input:
        r1 = "fastq/{sample}_1.fastq.gz",
        r2 = "fastq/{sample}_2.fastq.gz",
        index = "/home/rstudio/gencode.v36_salmon_1.4.0"
    output:
        "quants/{sample}/quant.sf"
    params:
        dir = "quants/{sample}"
    shell:
        "{SALMON} quant -i {input.index} -l A -p 6 --validateMappings \
         --gcBias --numGibbsSamples 20 -o {params.dir} \
         -1 {input.r1} -2 {input.r2}"
```

When used with

```
snakemake -j1 --snakefile snakemake_one
```

it takes 6 minutes to produce a folder `quants` with content
```
└── SRR1039508.1
    ├── aux_info
    │   ├── ambig_info.tsv
    │   ├── bootstrap
    │   │   ├── bootstraps.gz
    │   │   └── names.tsv.gz
    │   ├── expected_bias.gz
    │   ├── exp_gc.gz
    │   ├── fld.gz
    │   ├── meta_info.json
    │   ├── observed_bias_3p.gz
    │   ├── observed_bias.gz
    │   └── obs_gc.gz
    ├── cmd_info.json
    ├── lib_format_counts.json
    ├── libParams
    │   └── flenDist.txt
    ├── logs
    │   └── salmon_quant.log
    └── quant.sf
```

## Instrumentation of this process

We use the Rcollectl package (which must be installed from github.com/vjcitn/Rcollectl) to
monitor resource consumption while the snakemake process is running.

### Single sample 6 threads

Here is a display of the resource consumption for the single (paired-end) sample:

![single sample usage profile](https://storage.googleapis.com/bioc-anvil-images/airway_onesamp_collectl.png)

That run uses a setting of `-p 6` for `salmon quant`, which allows the algorithm to use 6 threads.   

### Single sample 12 threads

We
used a 16 core machine, and raised the value of `-p` to 12 to obtain the following profile:

![single sample usage, 12 threads](https://storage.googleapis.com/bioc-anvil-images/salmon_p12_one.png)

### Eight samples 12 threads

![eight samples 12 threads](https://storage.googleapis.com/bioc-anvil-images/salmon_p12_eight.png)

It takes about 40 minutes to do the eight samples, using about 80% of the available CPUs and about 16
GB RAM overall.

### Four samples 3 threads 4 processes (-j4 for snakemake)

Finally, we obtain some data to compare thread-based parallelism to process parallelism.  We'll
reduce the number of threads per sample, but run 4 samples at once via snakemake -j argument.

![four samples 3 threads -j4](https://storage.googleapis.com/bioc-anvil-images/salmon_p3_j4.png)

It seems the throughput is a bit better here, but we should check the comparability
of results of the different approaches before declaring this.

## Conclusions

Rcollectl can be used to obtain useful profiling information on AnVIL interactive work.  We saw
that the salmon quantification could be tuned to consume 80-90% of available CPUs.  The
RAM requirements and disk consumption are measurable.  It does not seem possible to decouple
the NCPU from the RAM elements in a fine-grained way in AnVIL interactive cloud environments.  For 16 cores we could
choose between 14.4 and 64 GB RAM.  64 GB seems too much for the task we are doing.

Additional work of interest: snakemake is used only because a working snakefile was published.  We
would like to be able to conduct these exercises in a more Bioconductor-native way.  Specifically,
we should have an object representing the read data in a self-describing way, and use BiocParallel
to manage the execution approach.  

Furthermore, as we define the self-describing representation of the read
data in Bioconductor, we should consider how it maps to the AnVIL/Terra-native
structures related to the workspace data tables, and DRS.

# test
# timestamp file name
cl_timestamp_file <- function(proc) {
  paste0(proc$target, "-", proc$node_name, "-", proc$date, ".timestamp.txt")
}

#' @rdname cl_timestamp
#' @title Functions to add time stamps to collectl output
#' @importFrom utils read.delim
#' @param proc an entity inheriting from "Rcollectl_process" S3 class
#' @param step character(1) name of step within a workflow
#' @return `cl_timestamp()` returns a tab delimited text file
#' @examples
#' id <- cl_start() 
#' Sys.sleep(2)
#' cl_timestamp(id, "step1")
#' Sys.sleep(2)
#' Sys.sleep(2)
#' cl_timestamp(id, "step2")
#' Sys.sleep(2)
#' Sys.sleep(2)
#' cl_timestamp(id, "step3")
#' Sys.sleep(2)
#' cl_stop(id)
#' @export
cl_timestamp <- function(proc, step) {
  con <- file(cl_timestamp_file(proc))
  open(con, "a")
  on.exit(close(con))
  text <- paste(step, Sys.time(), sep="\t")
  writeLines(text, con)
  invisible(NULL)
}

# read in collectl output data
cl_collectl_data <- function(arg) {
  if (inherits(arg, "Rcollectl_process")) {
    file <- cl_result_path(arg)
  } else {
    file <- arg
  }
  return(cl_parse(file))
}

# read in timestamp data
cl_timestamp_data <- function(arg, tz="EST") {
  if (inherits(arg, "Rcollectl_process")) {
    file <- cl_timestamp_file(arg)
  } else {
    file <- sub(".tab.gz$", ".timestamp.txt", arg)
  }
  timestamps <- read.delim(file, header = FALSE,
    col.names = c("Step", "sampdate"), sep = "\t")
  timestamps$sampdate <- as.POSIXct(timestamps$sampdate, tz=tz)
  return(timestamps)
}


#' @rdname cl_timestamp
#' @importFrom utils read.delim
#' @param arg proc (an entity inheriting from "Rcollectl_process" S3 class) or path to collectl output
#' @return `cl_timestamp_layer()` and `cl_timestamp_label()` return objects that can be combined with ggplot.
#' @examples
#' path <- cl_result_path(id)
#' plot_usage(cl_parse(path)) +
#'   cl_timestamp_layer(path) +
#'   cl_timestamp_label(path) +
#'   ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust=1))
#' @export
cl_timestamp_layer <- function(arg) {
  timestamps <- cl_timestamp_data(arg)
  geom_vline(xintercept = timestamps$sampdate, color = "blue")
}

#' @rdname cl_timestamp
#' @param tz character(1) time zone code
#' @export
cl_timestamp_label <- function(arg, tz="EST") {
  usage_df_all <- cl_collectl_data(arg)
  timestamps <- cl_timestamp_data(arg)

  xlabel_time <- as.numeric(round(
    difftime(
      as.POSIXct(timestamps$sampdate, tz=tz),
      as.POSIXct(usage_df_all[1, "sampdate"], tz=tz),
      units='mins')
  ))
  xlabel_time <- sprintf("%d:%02d", floor(xlabel_time / 60), xlabel_time %% 60)

  scale_x_continuous(
    breaks = as.POSIXct(timestamps$sampdate, tz=tz),
    labels = paste0(xlabel_time, "\n", timestamps$Step))
}

vizdf = function(x, tz="EST") {
 cpu_active = function(x)
   data.frame(tm = as.POSIXct(x$sampdate, tz=tz), xtype="CPU_MEM", 
     pos="top", value = 100-x$`CPU_Idle%`, type="%CPU active")
 mem_used = function(x)
   data.frame(tm = as.POSIXct(x$sampdate, tz=tz), xtype="CPU_MEM", 
     pos="bot", value = x$MEM_Used, type="MEM used")
 net_KB_cum = function(x)
   data.frame(tm = as.POSIXct(x$sampdate, tz=tz), xtype="NET_DSK",  
     pos="top", value = (x$NET_RxKBTot+x$NET_TxKBTot), type="KB NET")
 dsk_KBwr_cum = function(x)
   data.frame(tm = as.POSIXct(x$sampdate, tz=tz), xtype="NET_DSK", 
     pos="bot", value = cumsum(x$DSK_WriteKBTot), type="Cumul KB disk")
 
 rbind(cpu_active(x), mem_used(x), net_KB_cum(x), dsk_KBwr_cum(x))
}

#' elementary display of usage data from collectl
#' @import ggplot2
#' @param x output of cl_parse
#' @return ggplot with geom_point and facet_grid
#' @examples
#' lk = cl_parse(system.file("demotab/demo_1123.tab.gz", package="Rcollectl"))
#' plot_usage(lk)
#' @export
plot_usage = function(x) {
  ggplot(vizdf(x), aes(x=tm, y=value)) + geom_point() + 
          facet_grid(vars(type), scales="free")
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL ## Related to https://stat.ethz.ch/pipermail/bioc-devel/2020-April/016656.html
)

## ----vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE----------------
## Track time spent on making the vignette
startTime <- Sys.time()

## Bib setup
library("knitcitations")

## Load knitcitations with a clean bibliography
cleanbib()
cite_options(hyperlink = "to.doc", citation_format = "text", style = "html")

## Write bibliography information
bib <- c(
    R = citation(),
    BiocStyle = citation("BiocStyle")[1],
    knitcitations = citation("knitcitations")[1],
    knitr = citation("knitr")[1],
    rmarkdown = citation("rmarkdown")[1],
    sessioninfo = citation("sessioninfo")[1],
    testthat = citation("testthat")[1],
    Rcollectl = citation("Rcollectl")[1]
)

write.bibtex(bib, file = "Rcollectl.bib")

## ----"start", message=FALSE---------------------------------------------------
library("Rcollectl")

## ----lkdemo-------------------------------------------------------------------
lk = cl_parse(system.file("demotab/demo_1123.tab.gz", package="Rcollectl"))
dim(lk)
attr(lk, "meta")
lk[1:5,1:5]

## ----lkviz--------------------------------------------------------------------
plot_usage(lk)

## ----lklk,eval=FALSE----------------------------------------------------------
# id = cl_start([target file prefix])
# [use R until task to be measured is complete]
# cl_stop(id)
# usage_df = cl_parse(dir(patt=[target file prefix]))
# # analyze or filter the usage_df (for example, to trim away
# # time related to task delay or delay of `cl_stop`
# plot_usage(usage_df)

## ----lkts, eval=TRUE----------------------------------------------------------
     id <- cl_start() 
     Sys.sleep(2)
     #code
     cl_timestamp(id, "step1")
     Sys.sleep(2)
     # code
     Sys.sleep(2)
     cl_timestamp(id, "step2")
     Sys.sleep(2)
     # code
     Sys.sleep(2)
     cl_timestamp(id, "step3")
     Sys.sleep(2)
     # code
     cl_stop(id)
     path <- cl_result_path(id)
     
     plot_usage(cl_parse(path)) +
       cl_timestamp_layer(path) +
       cl_timestamp_label(path) +
       ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust=1))

## ----createVignette, eval=FALSE-----------------------------------------------
# ## Create the vignette
# library("rmarkdown")
# system.time(render("Rcollectl.Rmd", "BiocStyle::html_document"))
# 
# ## Extract the R code
# library("knitr")
# knit("Rcollectl.Rmd", tangle = TRUE)

## ----createVignette2----------------------------------------------------------
## Clean up
file.remove("Rcollectl.bib")

## ----reproduce1, echo=FALSE---------------------------------------------------
## Date the vignette was generated
Sys.time()

## ----reproduce2, echo=FALSE---------------------------------------------------
## Processing time in seconds
totalTime <- diff(c(startTime, Sys.time()))
round(totalTime, digits = 3)

## ----reproduce3, echo=FALSE-------------------------------------------------------------------------------------------
## Session info
library("sessioninfo")
options(width = 120)
session_info()

## ----vignetteBiblio, results = "asis", echo = FALSE, warning = FALSE, message = FALSE---------------------------------
## Print bibliography
bibliography()

#' parse a collectl output -- could be conditional on discovered call
#' @importFrom lubridate as_datetime
#' @importFrom utils browseURL read.delim
#' @param path character(1) path to (possibly gzipped) collectl output
#' @param tz character(1) POSIXct time zone code, defaults to "EST"
#' @param rescale_mem logical(1) if TRUE, divide reported MEM_Used by 1e6
#' @return a data.frame
#' @note A lubridate datetime is added as a column.  The test file `demo_1123.tab.gz` is
#' a collectl-generated report for a session ranging over 10 minutes, analyzing RNA-seq data
#' on a multicore machine.
#' @examples
#' lk = cl_parse(system.file("demotab/demo_1123.tab.gz", package="Rcollectl"))
#' head(lk)
#' @export
cl_parse = function(path, tz="EST", rescale_mem=TRUE) {
	full = readLines(path)
        inds = grep("^####", full)
        stopifnot(length(inds)==2)
        lastm = inds[2]
	meta = readLines(path)[seq_len(lastm)]
	dat = read.delim(path, skip=lastm, check.names=FALSE, sep=" ")
	names(dat) = gsub("\\[(...)\\]", "\\1_", names(dat))
        dat = revise_date(dat, tz=tz)
        if (rescale_mem) dat$MEM_Used = dat$MEM_Used/1e6
	attr(dat, "meta") = meta
	dat
}

revise_date = function(x, tz) {
 c2 = paste(x[,1], x[,2])
 pred = gsub("(....)(..)(..)(.*)", "\\1-\\2-\\3\\4", c2)
 x$sampdate = lubridate::as_datetime(pred, tz=tz)
 x
}

#' check for collectl availability
#' @return logical(1)
#' @examples
#' cl_exists()
#' @export
cl_exists = function() {
  x = try(system2("collectl", "--help", stdout=TRUE))
  if (!inherits(x, "try-error") && length(x)>30) return(TRUE)
  FALSE
}

#' start collectl if possible
#' @importFrom processx process
#' @param target character(1) path; destination of collectl report
#' @return instance of `Rcollectl_process` with components `process` (a processx R6 instance) and
#' `target` (a file path where collectl results will be written)
#' @examples
#' if (cl_exists()) {
#'   zz = cl_start()
#'   Sys.sleep(2)
#'   print(zz)
#'   Sys.sleep(2)
#'   print(cl_result_path(zz))
#'   cl_stop(zz)
#'   Sys.sleep(2)
#'   zz$process$is_alive()
#' }
#' @export
cl_start = function(target = tempfile()) {
 proc = try(processx::process$new("collectl", args=c("-scdmn", "-P", paste("-f", target, sep=""))))
 ans = list(process=proc, target=target, node_name=Sys.info()[["nodename"]],
  date=format(Sys.Date(), "%Y%m%d"))
 class(ans) = "Rcollectl_process"
 ans
}

#' print method for Rcollectl process
#' @param x an entity inheriting from "Rcollectl_process" S3 class
#' @param \dots not used
#' @return invisibly returns the input
#' @examples
#' example(cl_start)
#' @export
print.Rcollectl_process = function(x, ...) {
 cat("Rcollectl process object\n  ")
 print(x$process)
 cat("  ")
 cat(x$target, "\n")
 invisible(x)
}

#' stop collectl via processx interrupt
#' @param proc an entity inheriting from "Rcollectl_process" S3 class
#' @return invisibly returns the input
#' @examples
#' example(cl_start)
#' @export
cl_stop = function(proc) {
 stopifnot(inherits(proc, "Rcollectl_process"))
 proc$process$interrupt()
 invisible(proc)
}

#' get full path to collectl report
#' @param proc an entity inheriting from "Rcollectl_process" S3 class
#' @return character(1) path to report
#' @examples
#' example(cl_start)
#' @export
cl_result_path = function(proc) {
 stopifnot(inherits(proc, "Rcollectl_process"))
 dn = dirname(proc$target)
 paste0(proc$target, "-", proc$node_name, "-", proc$date, ".tab.gz")
}

library(Rcollectl)

test_that("cl_parse succeeds", {
 lk = cl_parse(system.file("demotab/demo_1123.tab.gz", package="Rcollectl"))
 expect_true(all(dim(lk)==c(478,71)))
 expect_true(length(grep("CPU", names(lk)))==21)
})

test_that("plot_usage succeeds", {
 lk = cl_parse(system.file("demotab/demo_1123.tab.gz", package="Rcollectl"))
 x = plot_usage(lk)
 expect_true("ggplot" %in% class(x))
 expect_true("FacetGrid" %in% class(x$facet))
})

test_that("cl_start/stop succeed", {
 if (cl_exists()) {
  lk = cl_start()
  expect_true(lk$process$is_alive())
  cl_stop(lk)
  Sys.sleep(2)
  expect_false(lk$process$is_alive())
  }
})

test_that("it works", {
 if (cl_exists()) {
  lk = cl_start()
  x = rnorm(2e7)
  Sys.sleep(8)
  x = rnorm(2e7)
  cl_stop(lk)
  targ = cl_result_path(lk)
  expect_true(file.exists(targ))
  p = cl_parse(targ)
  expect_true(nrow(p)>=5)  # at sleep of 5, had 4 rows!
  }
})

#' browse a page defining units of collectl reporting
#' @param \dots passed to \link{browseURL}
#' @return side effect is running browseURL
#' @examples
#' if (interactive()) {
#' browse_units()
#' }
#' @export
browse_units = function(...) {
	 path = system.file("units", "cl_units.html", package="Rcollectl")
	 browseURL(paste0("file://", path), ...)
}

---
title: "Rcollectl"
author: 
  - name: Vince Carey
    affiliation:
    - Harvard Medical School
    email: stvjc@channing.harvard.edu
  - name: Yubo Cheng
    affiliation:
    - Roswell Park Comprehensive Cancer Center
    email: yubo.cheng@roswellpark.org
output: 
  BiocStyle::html_document:
    self_contained: yes
    toc: true
    toc_float: true
    toc_depth: 2
    code_folding: show
date: "`r doc_date()`"
package: "`r pkg_ver('Rcollectl')`"
vignette: >
  %\VignetteIndexEntry{Rcollectl}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL ## Related to https://stat.ethz.ch/pipermail/bioc-devel/2020-April/016656.html
)
```


```{r vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE}
## Track time spent on making the vignette
startTime <- Sys.time()

## Bib setup
library("knitcitations")

## Load knitcitations with a clean bibliography
cleanbib()
cite_options(hyperlink = "to.doc", citation_format = "text", style = "html")

## Write bibliography information
bib <- c(
    R = citation(),
    BiocStyle = citation("BiocStyle")[1],
    knitcitations = citation("knitcitations")[1],
    knitr = citation("knitr")[1],
    rmarkdown = citation("rmarkdown")[1],
    sessioninfo = citation("sessioninfo")[1],
    testthat = citation("testthat")[1],
    Rcollectl = citation("Rcollectl")[1]
)

write.bibtex(bib, file = "Rcollectl.bib")
```

# Quick start for `Rcollectl`

```{r "start", message=FALSE}
library("Rcollectl")
```
[Collectl](https://collectl.sourceforge.net/) is
a unix-based tool that will perform measurements on system resource 
consumption of various types.  We provide a demonstration
output with the package:

```{r lkdemo}
lk = cl_parse(system.file("demotab/demo_1123.tab.gz", package="Rcollectl"))
dim(lk)
attr(lk, "meta")
lk[1:5,1:5]
```


```{r lkviz}
plot_usage(lk)
```

From this display, we can see that about a burst of network activity
around 14:43 is followed by consumption of CPU, memory, and disk resources.
The % CPU active never exceeds 30, memory consumption started relatively
high when sampling began, growing to about 15.5 GB. and 250MB were written
to disk over the entire interval.

To generate a display like this, we use commands shown below.  You can
use an arbitrary string as `[target file prefix]`.  Thus `cl_start("foo")`
will produce a file foo-[hostname]-[yyyymmdd].tab.gz, containing timing
and consumption data, where [hostname] is the value of `hostname` and
[yyyymmdd] is a representation of the current date.  Use different target
file prefixes for runs you wish to distinguish.

```{r lklk,eval=FALSE}
id = cl_start([target file prefix])
[use R until task to be measured is complete]
cl_stop(id)
usage_df = cl_parse(dir(patt=[target file prefix]))
# analyze or filter the usage_df (for example, to trim away
# time related to task delay or delay of `cl_stop`
plot_usage(usage_df)
```

# Timestamps

Yubo Cheng has added functionality allowing us to annotate
usage plots with labels related to task phases.  Here is
the code from the example showing how to introduce annotations
in the time profile.

```{r lkts, eval=TRUE}
     id <- cl_start() 
     Sys.sleep(2)
     #code
     cl_timestamp(id, "step1")
     Sys.sleep(2)
     # code
     Sys.sleep(2)
     cl_timestamp(id, "step2")
     Sys.sleep(2)
     # code
     Sys.sleep(2)
     cl_timestamp(id, "step3")
     Sys.sleep(2)
     # code
     cl_stop(id)
     path <- cl_result_path(id)
     
     plot_usage(cl_parse(path)) +
       cl_timestamp_layer(path) +
       cl_timestamp_label(path) +
       ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust=1))
```


# Reproducibility

The `r Biocpkg("Rcollectl")` package `r citep(bib[["Rcollectl"]])` was made possible thanks to:

* R `r citep(bib[["R"]])`
* `r Biocpkg("BiocStyle")` `r citep(bib[["BiocStyle"]])`
* `r CRANpkg("knitcitations")` `r citep(bib[["knitcitations"]])`
* `r CRANpkg("knitr")` `r citep(bib[["knitr"]])`
* `r CRANpkg("rmarkdown")` `r citep(bib[["rmarkdown"]])`
* `r CRANpkg("sessioninfo")` `r citep(bib[["sessioninfo"]])`
* `r CRANpkg("testthat")` `r citep(bib[["testthat"]])`

This package was developed using `r BiocStyle::Githubpkg("lcolladotor/biocthis")`.


Code for creating the vignette

```{r createVignette, eval=FALSE}
## Create the vignette
library("rmarkdown")
system.time(render("Rcollectl.Rmd", "BiocStyle::html_document"))

## Extract the R code
library("knitr")
knit("Rcollectl.Rmd", tangle = TRUE)
```

```{r createVignette2}
## Clean up
file.remove("Rcollectl.bib")
```

Date the vignette was generated.

```{r reproduce1, echo=FALSE}
## Date the vignette was generated
Sys.time()
```

Wallclock time spent generating the vignette.

```{r reproduce2, echo=FALSE}
## Processing time in seconds
totalTime <- diff(c(startTime, Sys.time()))
round(totalTime, digits = 3)
```

`R` session information.

```{r reproduce3, echo=FALSE}
## Session info
library("sessioninfo")
options(width = 120)
session_info()
```



# Bibliography

This vignette was generated using `r Biocpkg("BiocStyle")` `r citep(bib[["BiocStyle"]])`
with `r CRANpkg("knitr")` `r citep(bib[["knitr"]])` and `r CRANpkg("rmarkdown")` `r citep(bib[["rmarkdown"]])` running behind the scenes.

Citations made with `r CRANpkg("knitcitations")` `r citep(bib[["knitcitations"]])`.

```{r vignetteBiblio, results = "asis", echo = FALSE, warning = FALSE, message = FALSE}
## Print bibliography
bibliography()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/viz.R
\name{plot_usage}
\alias{plot_usage}
\title{elementary display of usage data from collectl}
\usage{
plot_usage(x)
}
\arguments{
\item{x}{output of cl_parse}
}
\value{
ggplot with geom_point and facet_grid
}
\description{
elementary display of usage data from collectl
}
\examples{
lk = cl_parse(system.file("demotab/demo_1123.tab.gz", package="Rcollectl"))
plot_usage(lk)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process.R
\name{cl_result_path}
\alias{cl_result_path}
\title{get full path to collectl report}
\usage{
cl_result_path(proc)
}
\arguments{
\item{proc}{an entity inheriting from "Rcollectl_process" S3 class}
}
\value{
character(1) path to report
}
\description{
get full path to collectl report
}
\examples{
example(cl_start)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timestamp.R
\name{cl_timestamp}
\alias{cl_timestamp}
\alias{cl_timestamp_layer}
\alias{cl_timestamp_label}
\title{Functions to add time stamps to collectl output}
\usage{
cl_timestamp(proc, step)

cl_timestamp_layer(arg)

cl_timestamp_label(arg, tz = "EST")
}
\arguments{
\item{proc}{an entity inheriting from "Rcollectl_process" S3 class}

\item{step}{character(1) name of step within a workflow}

\item{arg}{proc (an entity inheriting from "Rcollectl_process" S3 class) or path to collectl output}

\item{tz}{character(1) time zone code}
}
\value{
\code{cl_timestamp()} returns a tab delimited text file

\code{cl_timestamp_layer()} and \code{cl_timestamp_label()} return objects that can be combined with ggplot.
}
\description{
Functions to add time stamps to collectl output
}
\examples{
id <- cl_start() 
Sys.sleep(2)
cl_timestamp(id, "step1")
Sys.sleep(2)
Sys.sleep(2)
cl_timestamp(id, "step2")
Sys.sleep(2)
Sys.sleep(2)
cl_timestamp(id, "step3")
Sys.sleep(2)
cl_stop(id)
path <- cl_result_path(id)
plot_usage(cl_parse(path)) +
  cl_timestamp_layer(path) +
  cl_timestamp_label(path) +
  ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust=1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cl_parse.R
\name{cl_parse}
\alias{cl_parse}
\title{parse a collectl output -- could be conditional on discovered call}
\usage{
cl_parse(path, tz = "EST", rescale_mem = TRUE)
}
\arguments{
\item{path}{character(1) path to (possibly gzipped) collectl output}

\item{tz}{character(1) POSIXct time zone code, defaults to "EST"}

\item{rescale_mem}{logical(1) if TRUE, divide reported MEM_Used by 1000}
}
\value{
a data.frame
}
\description{
parse a collectl output -- could be conditional on discovered call
}
\note{
A lubridate datetime is added as a column.  The test file \code{demo_1123.tab.gz} is
a collectl-generated report for a session ranging over 10 minutes, analyzing RNA-seq data
on a multicore machine.
}
\examples{
lk = cl_parse(system.file("demotab/demo_1123.tab.gz", package="Rcollectl"))
head(lk)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process.R
\name{print.Rcollectl_process}
\alias{print.Rcollectl_process}
\title{print method for Rcollectl process}
\usage{
\method{print}{Rcollectl_process}(x, ...)
}
\arguments{
\item{x}{an entity inheriting from "Rcollectl_process" S3 class}

\item{\dots}{not used}
}
\value{
invisibly returns the input
}
\description{
print method for Rcollectl process
}
\examples{
example(cl_start)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browse_units.R
\name{browse_units}
\alias{browse_units}
\title{browse a page defining units of collectl reporting}
\usage{
browse_units(...)
}
\arguments{
\item{\dots}{passed to \link{browseURL}}
}
\value{
side effect is running browseURL
}
\description{
browse a page defining units of collectl reporting
}
\examples{
if (interactive()) {
browse_units()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process.R
\name{cl_stop}
\alias{cl_stop}
\title{stop collectl via processx interrupt}
\usage{
cl_stop(proc)
}
\arguments{
\item{proc}{an entity inheriting from "Rcollectl_process" S3 class}
}
\value{
invisibly returns the input
}
\description{
stop collectl via processx interrupt
}
\examples{
example(cl_start)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process.R
\name{cl_start}
\alias{cl_start}
\title{start collectl if possible}
\usage{
cl_start(target = tempfile())
}
\arguments{
\item{target}{character(1) path; destination of collectl report}
}
\value{
instance of \code{Rcollectl_process} with components \code{process} (a processx R6 instance) and
\code{target} (a file path where collectl results will be written)
}
\description{
start collectl if possible
}
\examples{
if (cl_exists()) {
  zz = cl_start()
  Sys.sleep(2)
  print(zz)
  Sys.sleep(2)
  print(cl_result_path(zz))
  cl_stop(zz)
  Sys.sleep(2)
  zz$process$is_alive()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process.R
\name{cl_exists}
\alias{cl_exists}
\title{check for collectl availability}
\usage{
cl_exists()
}
\value{
logical(1)
}
\description{
check for collectl availability
}
\examples{
cl_exists()
}
